@@ -29,7 +29,7 @@ gclue_service_location_initable_iface_init (GInitableIface *iface);
 
 G_DEFINE_TYPE_WITH_CODE (GClueServiceLocation,
                          gclue_service_location,
-                         GCLUE_TYPE_LOCATION_SKELETON,
+                         GCLUE_DBUS_TYPE_LOCATION_SKELETON,
                          G_IMPLEMENT_INTERFACE (G_TYPE_INITABLE,
                                                 gclue_service_location_initable_iface_init));
 
@@ -72,7 +72,6 @@ gclue_service_location_get_property (GObject    *object,
                                      GParamSpec *pspec)
 {
         GClueServiceLocation *self = GCLUE_SERVICE_LOCATION (object);
-        GClueLocation *location = GCLUE_LOCATION (object);
 
         switch (prop_id) {
         case PROP_CLIENT_INFO:
@@ -89,15 +88,21 @@ gclue_service_location_get_property (GObject    *object,
 
         case PROP_LOCATION:
         {
-                GeocodeLocation *loc;
+                GClueDBusLocation *location;
+                GClueLocation *loc;
                 gdouble altitude;
 
-                loc = geocode_location_new_with_description
-                        (gclue_location_get_latitude (location),
-                         gclue_location_get_longitude (location),
-                         gclue_location_get_accuracy (location),
-                         gclue_location_get_description (location));
-                altitude = gclue_location_get_altitude (location);
+                location = GCLUE_DBUS_LOCATION (object);
+                loc = gclue_location_new_with_description
+                        (gclue_dbus_location_get_latitude (location),
+                         gclue_dbus_location_get_longitude (location),
+                         gclue_dbus_location_get_accuracy (location),
+                         gclue_dbus_location_get_description (location));
+                gclue_location_set_speed
+                        (loc, gclue_dbus_location_get_speed (location));
+                gclue_location_set_heading
+                        (loc, gclue_dbus_location_get_heading (location));
+                altitude = gclue_dbus_location_get_altitude (location);
                 if (altitude != GEOCODE_LOCATION_ALTITUDE_UNKNOWN)
                         g_object_set (loc, "altitude", altitude, NULL);
 
@@ -117,7 +122,6 @@ gclue_service_location_set_property (GObject      *object,
                                      GParamSpec   *pspec)
 {
         GClueServiceLocation *self = GCLUE_SERVICE_LOCATION (object);
-        GClueLocation *location = GCLUE_LOCATION (object);
 
         switch (prop_id) {
         case PROP_CLIENT_INFO:
@@ -134,21 +138,29 @@ gclue_service_location_set_property (GObject      *object,
 
         case PROP_LOCATION:
         {
-                GeocodeLocation *loc;
+                GClueDBusLocation *location;
+                GClueLocation *loc;
                 gdouble altitude;
+                GeocodeLocation *g_loc;
 
+                location = GCLUE_DBUS_LOCATION (object);
                 loc = g_value_get_object (value);
-                gclue_location_set_latitude
-                        (location, geocode_location_get_latitude (loc));
-                gclue_location_set_longitude
-                        (location, geocode_location_get_longitude (loc));
-                gclue_location_set_accuracy
-                        (location, geocode_location_get_accuracy (loc));
-                gclue_location_set_description
-                        (location, geocode_location_get_description (loc));
-                altitude = geocode_location_get_altitude (loc);
+                g_loc = GEOCODE_LOCATION (loc);
+                gclue_dbus_location_set_latitude
+                        (location, geocode_location_get_latitude (g_loc));
+                gclue_dbus_location_set_longitude
+                        (location, geocode_location_get_longitude (g_loc));
+                gclue_dbus_location_set_accuracy
+                        (location, geocode_location_get_accuracy (g_loc));
+                gclue_dbus_location_set_description
+                        (location, geocode_location_get_description (g_loc));
+                gclue_dbus_location_set_speed
+                        (location, gclue_location_get_speed (loc));
+                gclue_dbus_location_set_heading
+                        (location, gclue_location_get_heading (loc));
+                altitude = geocode_location_get_altitude (g_loc);
                 if (altitude != GEOCODE_LOCATION_ALTITUDE_UNKNOWN)
-                        gclue_location_set_altitude (location, altitude);
+                        gclue_dbus_location_set_altitude (location, altitude);
                 break;
         }
 
@@ -172,10 +184,10 @@ gclue_service_location_handle_method_call (GDBusConnection       *connection,
         GDBusInterfaceVTable *skeleton_vtable;
 
         if (!gclue_client_info_check_bus_name (priv->client_info, sender)) {
-                g_dbus_method_invocation_return_error (invocation,
-                                                       G_DBUS_ERROR,
-                                                       G_DBUS_ERROR_ACCESS_DENIED,
-                                                       "Access denied");
+                g_dbus_method_invocation_return_error_literal (invocation,
+                                                               G_DBUS_ERROR,
+                                                               G_DBUS_ERROR_ACCESS_DENIED,
+                                                               "Access denied");
                 return;
         }
 
@@ -320,7 +332,7 @@ gclue_service_location_class_init (GClueServiceLocationClass *klass)
         gParamSpecs[PROP_LOCATION] = g_param_spec_object ("location",
                                                           "Location",
                                                           "Location",
-                                                          GEOCODE_TYPE_LOCATION,
+                                                          GCLUE_TYPE_LOCATION,
                                                           G_PARAM_READWRITE);
         g_object_class_install_property (object_class,
                                          PROP_LOCATION,
@@ -333,8 +345,8 @@ gclue_service_location_init (GClueServiceLocation *location)
         location->priv = G_TYPE_INSTANCE_GET_PRIVATE (location,
                                                       GCLUE_TYPE_SERVICE_LOCATION,
                                                       GClueServiceLocationPrivate);
-        gclue_location_set_altitude (GCLUE_LOCATION (location),
-                                     GEOCODE_LOCATION_ALTITUDE_UNKNOWN);
+        gclue_dbus_location_set_altitude (GCLUE_DBUS_LOCATION (location),
+                                          GEOCODE_LOCATION_ALTITUDE_UNKNOWN);
 }
 
 static gboolean
@@ -359,7 +371,7 @@ GClueServiceLocation *
 gclue_service_location_new (GClueClientInfo *info,
                             const char      *path,
                             GDBusConnection *connection,
-                            GeocodeLocation *location,
+                            GClueLocation   *location,
                             GError         **error)
 {
         return g_initable_new (GCLUE_TYPE_SERVICE_LOCATION,
